#ifndef CNEURALGNATS_H
#define CNEURALGNATS_H

//------------------------------------------------------------------------
//
//	Name: CNeuralGnats.h
//
//  Code by: George Taylor 2013
//
//  Based on a framework provided by: Mat Buckland 2002
//
//  Desc: Class to create a NeuralGnats object 
//
//------------------------------------------------------------------------
#include <vector>
#include <math.h>

#include "CNeuralNet.h"
#include "utils.h"
#include "C2DMatrix.h"
#include "SVector2D.h"
#include "CParams.h"

#define MAX_LINES 4
#define LEFTHANDED 1
#define RIGHTHANDED -1
#define AMBIVALENT 0


using namespace std;

typedef struct 
{
 double x1, y1;
 double x2, y2;
}LINE, *LPLINE;


class CNeuralGnat
{

private:

 // The NeuralGnat's neural net
 CNeuralNet		m_ItsBrain;

 SVector2D		m_vWorldPosition;
 SVector2D		m_vTranslatedPosition;
 SVector2D		m_vTranslatedFungusPosition;
 SVector2D      m_vDisplayedGnatPos;
 SVector2D      m_vDisplayedFungusPos;

 double         m_ClosestFungusAngle;
 double         m_ClosestGnatAngle;
 double         m_dDistanceToFungus;
 double         m_dDistanceToGnat;
 double         m_dDistanceGnatToCentre;
 double			m_dPrevRotation;
 double			m_dRotation;
 double			m_dFitness;
 double			m_dScale;
 
 int            m_iClosestFungus;
 int            m_iClosestGnat;
 int            m_iMyIndex;
 int            m_iHandedness;
 int            m_iFrozen;

 bool        PointOnLine(LINE const &line, SVector2D const &vPoint);
 bool        FindIntersectionPoint(LINE const &line, SVector2D const &vPosition, SVector2D const &vProjection, SVector2D *pvIntersectionPoint);
 void  	     GetClosestFungus(vector<SVector2D> &fungi, double *pDistanceFromCentre);
 void        GetClosestGnat(vector<CNeuralGnat> &gnats);
 void        FindAnotherCloseFungus(vector<SVector2D> &fungi);
 void        GetFungusAwayFromWall(vector<SVector2D> &fungi);
 bool        PathToObjectIntersectsWall(SVector2D &vObjectWorldPos, double dDistanceObjectToCentre);
 double      MagnitudeAngularDifferences(double dAngleA, double dAngleB); 
 double      ReverseMagnitudeAngularDifferences(double dAngleA, double dMagnitudeAngularDifferences); 
 bool        HaveCollidingObjectives(SVector2D &vNeighbourTranslatedFungusPos);
 void        UpdatePosition(double X, double Y);
 void        CheckBounds(double X, double Y);
 void        CheckFrozenBounds(double X, double Y);



public:

	CNeuralGnat(SVector2D vStartPos, int arrayIndex);
	// ~CNeuralGnat();
	
	// updates the ANN with information from the gnats enviroment
	bool            Update(vector<SVector2D> &fungi, vector<CNeuralGnat> &gnats, bool bShow);


    void            GetDisplayedPoints(int *pX1, int *pY1, int *pX2, int *pY2);
	int             GetHandedness(void){return m_iHandedness;};
    void            GetLineAcross(int *pX1, int *pY1, int *pX2, int *pY2);
    void			WorldTransform(vector<SPoint> &gnat, double dScale);

  // checks to see if the NeuralGnat has 'collected' a fungus
  int       CheckForFungus(vector<SVector2D> &fungi, double *pDistanceToNeighbour);
  void		Reset(SVector2D vStartPos);
  

	//-------------------accessor functions
	SVector2D	Position()const{return m_vWorldPosition;}
	SVector2D	TranslatedPosition()const{return m_vTranslatedPosition;}
	void			IncrementFitness(){++m_dFitness;}
	void			DecrementFitness(){if(m_dFitness > 0)--m_dFitness;}

	double		Fitness()const{return m_dFitness;}
//	double		Rotation()const{return m_dRotation;}
	double		DistanceToCentre(){return m_dDistanceGnatToCentre;};
  	int		    ClosestGnatIndex()const{return m_iClosestGnat;}
  	int		    ClosestFungusIndex()const{return m_iClosestFungus;}
  void         PutWeights(vector<double> &w){m_ItsBrain.PutWeights(w);}

  int       GetNumberOfWeights()const{return m_ItsBrain.GetNumberOfWeights();}
};


#endif

	
	